// edit the config.h tab and enter your Adafruit IO credentials
// and any additional configuration needed for WiFi, cellular,
// or ethernet clients.

#include "config.h"

#include "Adafruit_Sensor.h"
#include "Adafruit_AM2320.h"

//Set up your Adafruit IO feeds for sensors
AdafruitIO_Feed *am2320_hum = io.feed("am2320_hum");
AdafruitIO_Feed *am2320_temp = io.feed("am2320_temp");
AdafruitIO_Feed *lux = io.feed("lux");
AdafruitIO_Feed *soil_moisture = io.feed("soil_moisture");
//AdafruitIO_Feed *temp_probe0 = io.feed("temp_probe0");
AdafruitIO_Feed *temp_probe1 = io.feed("temp_probe1");
AdafruitIO_Feed *temp_probe2 = io.feed("temp_probe2");
AdafruitIO_Feed *temp_probe3 = io.feed("temp_probe3");
//Set up IO feeds for toggles
AdafruitIO_Feed *fansControl = io.feed("fansControl");
AdafruitIO_Feed *lightControl = io.feed("lightControl");
AdafruitIO_Feed *ledFanControl = io.feed("ledFanControl");
AdafruitIO_Feed *pumpControl = io.feed("pumpControl");


int fan_count = 0;
int light_on_off = 0;

int count = 0;

int tempPin0 = A0;
int tempPin1 = A9;
int tempPin2 = A2;
int tempPin3 = A3;
int tempDelay = 100;

int lightPin = A4;
int soilPin = A7;

#define lightRelayPin 25
#define pumpRelayPin 15
#define ledFanRelayPin 4
#define fansRelayPin 14



float rawRange = 4096; // 3.3v
float logRange = 5.0; // 3.3v = 10^5 lux

Adafruit_AM2320 am2320 = Adafruit_AM2320();


void setup() {
  //Setup toggle pin outputs
  pinMode(lightRelayPin, OUTPUT);
  pinMode(pumpRelayPin, OUTPUT);
  pinMode(ledFanRelayPin, OUTPUT);
  pinMode(fansRelayPin, OUTPUT);

  // initialize serial communications at 9600 bps:
  Serial.begin(115200);

  while (!Serial) {
    delay(10); // hang out until serial port opens
  }

  //Connect to Adafruit.io
  Serial.print("Connecting to Adafruit IO");
  io.connect();

  //Setup handlers for toggle messages

  fansControl->onMessage(fansToggle);
  lightControl->onMessage(lightToggle);
  ledFanControl->onMessage(ledFanToggle);
  pumpControl->onMessage(pumpToggle);


  //  // wait for a connection
  while (io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  //
  //  // we are connected
  Serial.println();
  Serial.println(io.statusText());

  Serial.println("Connected!");

  while (!Serial) {
    delay(10); // hang out until serial port opens
  }


  Serial.println("Adafruit AM2320 Basic Test");
  am2320.begin();

  pinMode(13, OUTPUT);

}

void loop() {

  io.run();
  digitalWrite(13, HIGH);   // turn the LED on as a heartbeat for the sketch

  Serial.println(count);
  count++;

  //Read the light sensor value and print it to serial after converting to Lux
  int rawValue = analogRead(lightPin);
  //Serial.print("Raw = ");
  //Serial.print(rawValue);
  //Serial.print("Lux = ");
  float lux_reading = RawToLux(rawValue);
  Serial.print(lux_reading); Serial.print("; ");
  //delay(500);


  //Read the soil moisture sensor and print it to serial
  int soilValue = analogRead(soilPin);
  delay(tempDelay);
  soilValue = analogRead(soilPin);
  delay(tempDelay);
  soilValue = analogRead(soilPin);
  //Serial.print("soil = ");
  Serial.print(soilValue);
  Serial.print("; ");
  //delay(500);

  //Read the temperature and humididity from the AM2320 and print it to serial
  float temp = am2320.readTemperature();
  float hum = am2320.readHumidity();
  //Serial.print("Temp from AM2320: "); Serial.println(temp);

  temp = ((temp * 9.0 / 5.0) + 32.0);
  Serial.print(hum); Serial.print("; ");
  //Serial.print("Humidity from AM2320: "); Serial.println(hum);
  Serial.print(temp); Serial.print("; ");
  //delay(500);

  //Read each of the four temperature sensors three times throwing out the first two measurements and delaying a bit before each

  //Temp0 reading on A0
  int tempReading0 = analogRead(tempPin0);
  delay(tempDelay);
  tempReading0 = analogRead(tempPin0);
  delay(tempDelay);
  tempReading0 = analogRead(tempPin0);
  delay(tempDelay);

  //Serial.print("Temp0 = ");
  //Serial.print(tempReading0); Serial.print("; ");
  //Serial.println(tempReading0);

  //Temp1 reading on A1
  int tempReading1 = analogRead(tempPin1);
  delay(tempDelay);
  tempReading1 = analogRead(tempPin1);
  delay(tempDelay);
  tempReading1 = analogRead(tempPin1);
  delay(tempDelay);

  //Serial.print("Temp1 = ");
  //Serial.print(tempReading1); Serial.print("; ");
  //Serial.println(tempReading1);

  //Temp2 reading on A2
  int tempReading2 = analogRead(tempPin2);
  delay(tempDelay);
  tempReading2 = analogRead(tempPin2);
  delay(tempDelay);
  tempReading2 = analogRead(tempPin2);
  delay(tempDelay);

  //Serial.print("Temp2 = ");
  //Serial.print(tempReading2); Serial.print("; ");

  //Serial.println(tempReading2);

  digitalWrite(13, LOW);   // turn the LED on (HIGH is the voltage level)

  //Temp3 reading on A3
  int tempReading3 = analogRead(tempPin3);
  delay(tempDelay);
  tempReading3 = analogRead(tempPin3);
  delay(tempDelay);
  tempReading3 = analogRead(tempPin3);
  delay(tempDelay);

  //Serial.print("Temp3 = ");
  // Serial.print(tempReading3); Serial.print("; ");

  //Serial.println(tempReading3);


  // converting the 3rd reading from each temperature sensor to voltage, for 3.3v arduino use 3.3
  float voltage0 = tempReading0 * 5;
  voltage0 = voltage0 / 1024.0;

  float voltage1 = tempReading1 * 5;
  voltage1 = voltage1 / 1024.0;

  float voltage2 = tempReading2 * 5;
  voltage2 = voltage2 / 1024.0;

  float voltage3 = tempReading3 * 5;
  voltage3 = voltage3 / 1024.0;

  // print out the voltages
  //  Serial.print("V0 = "); Serial.println(voltage0);
  //  Serial.print("V1 = "); Serial.println(voltage1);
  //  Serial.print("V2 = "); Serial.println(voltage2);
  //  Serial.print("V3 = "); Serial.println(voltage3);
  //
  //  Serial.print(voltage0); Serial.print("; ");
  //  Serial.print(voltage1); Serial.print("; ");
  //  Serial.print(voltage2); Serial.print("; ");
  //  Serial.print(voltage3); Serial.print("; ");



  // now print out the temperature
  float temperature0C = (voltage0 - 0.5) * 10 ;
  float temperature1C = (voltage1 - 0.5) * 10 ;
  float temperature2C = (voltage2 - 0.5) * 10 ;
  float temperature3C = (voltage3 - 0.5) * 10 ;
  //
  //  //converting from 10 mv per degree wit 500 mV offset
  //  //to degrees ((voltage - 500mV) times 100)
  //  Serial.print("Temp0 = "); Serial.println(temperature0C);
  //  // now convert to Fahrenheit and print to serial
  //
  //  Serial.print(temperature0C); Serial.print("; ");
  //  Serial.print(temperature1C); Serial.print("; ");
  //  Serial.print(temperature2C); Serial.print("; ");
  //  Serial.print(temperature3C); Serial.println("; ");


  float temperature0F = (temperature0C * 9.0 / 5.0) + 32.0; Serial.print(temperature0F); Serial.print("; ");
  float temperature1F = (temperature1C * 9.0 / 5.0) + 32.0; Serial.print(temperature1F); Serial.print("; ");
  float temperature2F = (temperature2C * 9.0 / 5.0) + 32.0; Serial.print(temperature2F); Serial.print("; ");
  float temperature3F = (temperature3C * 9.0 / 5.0) + 32.0; Serial.print(temperature3F); Serial.println("; ");

  if (count == 8)
  {
    //write readings to Adafruit.io feeds every ~8 seconds

    am2320_hum->save(hum);
    am2320_temp->save(temp);
    lux->save(lux_reading);
    soil_moisture->save(soilValue);
    //temp_probe0->save(temperature0F);
    temp_probe1->save(temperature1F);
    temp_probe2->save(temperature2F);
    temp_probe3->save(temperature3F);

    digitalWrite(13, HIGH);   // flash the LED to show that data transmitted
    delay(100);
    digitalWrite(13, LOW);
    delay(100);


    digitalWrite(13, HIGH);
    delay(100);
    digitalWrite(13, LOW);
    delay(100);

    count = 0;
    Serial.println("Sent data to Adafruit.IO!");
  }


  //delay(500);
}

float RawToLux(int raw)
{
  float logLux = raw * logRange / rawRange;
  return pow(10, logLux);
}


// these functions are called whenever a 'digital' feed message
// is received from Adafruit IO. it was attached to
// the feeds in the setup() functions above.
//
//  fansControl->onMessage(fansToggle);
//  lightControl->onMessage(lightToggle);
//  ledFanControl->onMessage(ledFanToggle);
//  pumpControl->onMessage(pumpToggle);
//
//int lightRelayPin = 25;
//int pumpRelayPin = 15;
//int ledFanRelayPin = 4;
//int fansRelayPin = 14;


void fansToggle(AdafruitIO_Data *data) {

  Serial.print("received <- ");

  if (data->toPinLevel() == HIGH)
    Serial.println("Fans turned on");
  else
    Serial.println("Fans turned off");


  digitalWrite(fansRelayPin, data->toPinLevel());
}

void lightToggle(AdafruitIO_Data *data) {

  Serial.print("received <- ");

  if (data->toPinLevel() == HIGH)
    Serial.println("Lights turned on");
  else
    Serial.println("Lights turned off");


  digitalWrite(lightRelayPin, data->toPinLevel());
}

void ledFanToggle(AdafruitIO_Data *data) {

  Serial.print("received <- ");

  if (data->toPinLevel() == HIGH)
    Serial.println("Led fan turned on");
  else
    Serial.println("Led fan turned off");


  digitalWrite(ledFanRelayPin, data->toPinLevel());
}


void pumpToggle(AdafruitIO_Data *data) {

  Serial.print("received <- ");

  if (data->toPinLevel() == HIGH)
    Serial.println("Pump turned on");
  else
    Serial.println("Pump turned off");


  digitalWrite(pumpRelayPin, data->toPinLevel());
}
